@@ -120,7 +120,9 @@ static inline bool check_builtin_type(pTHX_ SV *val, BuiltinTypeID type_id) {
         case TYPE_ANY:
             return true;
         case TYPE_DEFINED:
-            return SvOK(val);
+            /* SvOK checks if defined, but be defensive for older Perls */
+            /* where constant 0 might have edge cases */
+            return SvOK(val) || SvIOK(val) || SvNOK(val) || SvPOK(val);
         case TYPE_STR:
             return SvOK(val) && !SvROK(val);  /* defined non-ref */
         case TYPE_INT:
@@ -195,7 +197,8 @@ static bool check_slot_type(pTHX_ SV *val, SlotSpec *spec) {
     if (spec->registered->perl_check) {
         dSP;
         int count;
-        bool result;
+        bool result = false;
+        SV *result_sv;
         ENTER;
         SAVETMPS;
         PUSHMARK(SP);
@@ -203,7 +206,11 @@ static bool check_slot_type(pTHX_ SV *val, SlotSpec *spec) {
         PUTBACK;
         count = call_sv(spec->registered->perl_check, G_SCALAR);
         SPAGAIN;
-        result = (count > 0) ? SvTRUE(POPs) : false;
+        if (count > 0) {
+            result_sv = POPs;
+            result = SvTRUE(result_sv);
+        }
+        PUTBACK;
         FREETMPS;
         LEAVE;
         return result;
@@ -1064,7 +1071,9 @@ static inline bool is_simple_op(OP *op) {
         case OP_GV:       /* *glob */
         case OP_GVSV:     /* $global */
         case OP_AELEMFAST:/* $array[const] */
+#if defined(OP_AELEMFAST_LEX) && OP_AELEMFAST_LEX != OP_AELEMFAST
         case OP_AELEMFAST_LEX:
+#endif
         case OP_NULL:     /* Often wraps simple ops */
             return true;
         default:
@@ -1728,11 +1737,36 @@ static XS(xs_is_locked) {
     XSRETURN_NO;
 }
 
+/* ============================================
+   Global cleanup
+   ============================================ */
+
+/* Cleanup during global destruction */
+static void object_cleanup_globals(pTHX_ void *data) {
+    PERL_UNUSED_ARG(data);
+
+    /* During global destruction, just NULL out pointers.
+     * Perl handles SV cleanup. Trying to free them ourselves
+     * can cause crashes due to destruction order. */
+    if (PL_dirty) {
+        g_type_registry = NULL;
+        g_class_registry = NULL;
+        g_func_accessor_registry = NULL;
+        return;
+    }
+
+    /* Normal cleanup - not during global destruction */
+    /* Note: Full cleanup omitted for simplicity; Perl handles SV refcounts */
+    g_type_registry = NULL;
+    g_class_registry = NULL;
+    g_func_accessor_registry = NULL;
+}
+
 /* ============================================
    Type Registry API
    ============================================ */
 
-/* C-level registration for external XS modules (called from BOOT) 
+/* C-level registration for external XS modules (called from BOOT)
    This is the fast path - no Perl callback overhead */
 PERL_CALLCONV void object_register_type_xs(pTHX_ const char *name, 
                                            ObjectTypeCheckFunc check,
@@ -2037,5 +2071,8 @@ XS_EXTERNAL(boot_object) {
     /* Singleton support */
     newXS("object::singleton", xs_singleton, __FILE__);
 
+    /* Register cleanup for global destruction */
+    Perl_call_atexit(aTHX_ object_cleanup_globals, NULL);
+
     Perl_xs_boot_epilog(aTHX_ ax);
 }
